%SoundSnipper
% GUI which enables 'snips' of a .wav file to be made. The snip can also be
% reversed and 'sped up' or 'slowed down.'
%
% LAST UPDATED by Andy French May 2014.
function varargout = soundsnipper(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @soundsnipper_OpeningFcn, ...
    'gui_OutputFcn',  @soundsnipper_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% GUI opening function
function soundsnipper_OpeningFcn(hObject, eventdata, handles, varargin)

%Define global arrays d (data) and h (handles) which will store data and be
%accesible to all functions
global d
global h

%Store handles array in h
h = handles;

%Set GUI defaults
set_gui_defaults
get_data_from_gui

%%

% GUI output function
function varargout = soundsnipper_OutputFcn(hObject, eventdata, handles)

%%

% Snip start time callback
function EDITtstart_Callback(hObject, eventdata, handles)
get_data_from_gui
update_snip_box

%%

% Snip stop time callback
function EDITtstop_Callback(hObject, eventdata, handles)
get_data_from_gui
update_snip_box

%%

% Snip ramp up and ramp down linear fade time
function EDITtfade_Callback(hObject, eventdata, handles)
get_data_from_gui

%%

% Time reverse snip checkbox
function CHECKreverse_Callback(hObject, eventdata, handles)
get_data_from_gui

%%

% Show graph editbox
function CHECKplot_Callback(hObject, eventdata, handles)
get_data_from_gui
update_graph

%%

% Save snip as a .wav file
function PUSHsnip_Callback(hObject, eventdata, handles)
global d
global h

%Display 'snipping ...' message to command window
set( h.TEXTinfo,'string','... Snipping waveform' )
drawnow

%Determine snip
nstart = max( floor( d.tstart*d.Fs ), 1 );
nstop = min( ceil( d.tstop*d.Fs ), length(d.left) );
s = [d.left,d.right];
s = s( nstart : nstop, : );

%Reverse sample?
if d.reverse == 1
    s = flipud(s);
end
dim = size(s);

%Interpolate if speed change is requested
if d.speed~=1
    s1 = interp1( (1:dim(1)).', s(:,1), linspace(1,dim(1), round( dim(1)/d.speed ) ).' );
    s2 = interp1( (1:dim(1)).', s(:,2), linspace(1,dim(1), round( dim(1)/d.speed ) ).' );
    s = [s1,s2];
    dim = size(s);
    clear s1 s2
end

%Define linear fade in and out
if d.duration/d.speed > 2*d.tfade
    ramp_up = linspace( 0, 1, floor( d.tfade*d.Fs/d.speed ) ).';
    ramp_down = linspace( 1, 0, floor( d.tfade*d.Fs/d.speed ) ).';
    fade = [ramp_up;ones( dim(1) - length(ramp_up) - length(ramp_down), 1); ramp_down ];
    fade = [fade,fade];
else
    %Not enough time to fade
    fade = ones(dim);
end

%Write snip
if strcmp( d.filename( end-3 : end ), '.mp3' )
    mp3write( fade.*s, d.Fs, strrep( d.filename,'.mp3','_snip.mp3') );
elseif strcmp( d.filename( end-3 : end ), '.wav' ) 
    wavwrite( fade.*s, d.Fs, strrep( d.filename,'.wav','_snip.wav') );
end
set( h.TEXTinfo,'string','Snip complete' );
clear s

%%

% Filename editbox
function EDITfilename_Callback(hObject, eventdata, handles)
get_data_from_gui

%%

% Load .wav file pushbutton
function PUSHloadwav_Callback(hObject, eventdata, handles)
global d
global h

[filename,pathname] = uigetfile({'*.mp3';'*.wav'},'Choose .wav or .mp3 file to snip');
if filename~=0
    
    %Get and store filename
    d.filename = [pathname,'\',filename];
    set( h.EDITfilename,'string',d.filename);
    
    %Load .wav file
    t0 = tic;
    set( h.TEXTinfo,'string',['Loading file: ',d.filename,' ...'] );
    drawnow
    if strcmp( d.filename( end-3 : end ), '.mp3' )
        [s,d.Fs,Nbits] = mp3read( d.filename );
    elseif strcmp( d.filename( end-3 : end ), '.wav' )
        [s,d.Fs,Nbits] = wavread( d.filename );
    end
    dim = size(s);
    if dim(2)==1
        %Convert to Stereo if .wav file is Mono
        d.left = s(:,1);
        d.right = d.left;
    else
        d.left = s(:,1);
        d.right = s(:,2);
        clear s
    end
    
    %Compute duration of waveform /s
    d.duration = dim(1)/d.Fs;
    
    %Set start and stop times
    d.tstart = 0;
    d.tstop = d.duration;
    set( h.EDITtstart, 'string', '0' );
    set( h.EDITtstop, 'string', num2str(d.duration) );
    
    %Plot graph
    update_graph
    
    %Update info box with loading time
    set( h.TEXTinfo,'string',['File: ',d.filename,' loaded in ',num2str(toc(t0)),' s.'] );
end

%%

% Reset tstart and tstop to correspond to full duration of waveform
function PUSHfullduration_Callback(hObject, eventdata, handles)
global d
global h
d.tstart = 0;
d.tstop = d.duration;
set( h.EDITtstart, 'string', '0' );
set( h.EDITtstop, 'string', num2str(d.duration) );
update_snip_box

%%

% Playback speed edit box
function EDITspeed_Callback(hObject, eventdata, handles)
get_data_from_gui

%%

% Set GUI defaults
function set_gui_defaults
global h
global d

%Initialise elements of data array
d.tstart = 0;
d.tstop = 10;
d.tfade = 2;
d.speed = 1;
d.Fs = 44000;
d.reverse = 0;
d.filename = [];
d.right = [];
d.left = [];
d.duration = [];
d.plot_graph = 0;

%Initialise GUI items
h.snip_box = [];
set( h.EDITtstart,'string',num2str( d.tstart ) );
set( h.EDITtstop,'string',num2str( d.tstop ) );
set( h.EDITtfade,'string',num2str( d.tfade ) );
set( h.EDITspeed,'string',num2str( d.speed ) );
set( h.TEXTinfo,'string','Welcome to SoundSnipper. A. French May 2014' );
set( h.EDITfilename,'string',' ' );
set( h.CHECKreverse,'value', d.reverse );
set( h.CHECKplot,'value', d.plot_graph );
axes( h.AXESwaveform ); cla; axis off;

%%

% Get data from GUI
function get_data_from_gui
global h
global d
d.tstart = str2num( get( h.EDITtstart, 'string' ) );
d.tstop = str2num( get( h.EDITtstop, 'string' ) );
d.tfade =  str2num( get( h.EDITtfade, 'string' ) );
d.speed = str2num( get( h.EDITspeed, 'string' ) );
d.reverse = get( h.CHECKreverse, 'value' );
d.filename = get( h.EDITfilename, 'string' );
d.plot_graph = get( h.CHECKplot, 'value' );

%%

% Update snip box
function update_snip_box
global h
global d
axes( h.AXESwaveform ); hold on;
ylimits = get(h.AXESwaveform,'ylim');
if ~isempty( h.snip_box )
    set( h.snip_box,'xdata',[ floor(d.tstart*d.Fs),floor(d.tstop*d.Fs),ceil(d.tstop*d.Fs),ceil(d.tstart*d.Fs) ],...
        'ydata',[ylimits(1),ylimits(1),ylimits(2),ylimits(2) ] );
end
axis off

%%

% Update Graph
function update_graph
global h
global d
axes( h.AXESwaveform );
cla
if d.plot_graph ~=0
    plot( d.left, 'b' ); hold on
    plot( d.right + 2, 'r' );
    axis tight
    
    %Plot snip region
    hold on;
    ylimits = get( h.AXESwaveform , 'ylim' );
    h.snip_box = patch([ floor(d.tstart*d.Fs),floor(d.tstop*d.Fs),ceil(d.tstop*d.Fs),ceil(d.tstart*d.Fs) ],...
        [ylimits(1),ylimits(1),ylimits(2),ylimits(2) ],'g','facealpha',0.7 );
    axis off
    drawnow
end

%End of code